# coding: utf-8
import logging
import time
import logstash

from .const import (USER, START_TIME_S, __version__,
                    LS_HOST, LS_PORT,
                    __PIP_PACKAGE_NAME__, CWD, SESSION_ID)

LOG_EXTRA = {
    'app_user': USER,
    'app_name': __PIP_PACKAGE_NAME__,
    'app_version': __version__,
    'app_reports': [],
    'app_is_debug_run': False,
    'app_event_type': None,
    'app_runtime_s': None,
    'app_workdir': CWD,
    'app_sessionid': SESSION_ID
}


def make_logger(extra, is_debug):
    logger = logging.getLogger()
    logger.setLevel(logging.DEBUG)

    if is_debug:
        file_fmt = '%(asctime)s - {} - %(name)s:{} - %(levelname)s - %(message)s'.format(
            extra['app_user'], __version__)
        f_handler = logging.FileHandler('ta-report.log')
        f_handler.setFormatter(logging.Formatter(file_fmt))
        logger.addHandler(f_handler)
        extra["app_is_debug_run"] = True
    else:
        ls_handler = logstash.TCPLogstashHandler(
            LS_HOST, LS_PORT, version=1, message_type="logstash")
        ls_handler.setLevel(logging.INFO)
        logger.addHandler(ls_handler)

    def level_logger(level, msg, event_type=None, **kwargs):
        extra["app_event_type"] = event_type
        extra["app_runtime_s"] = time.time() - START_TIME_S
        logger.log(level, msg, extra=extra, **kwargs)

    return level_logger
