# coding: utf-8
import logging
import pandas as pd
from .const import CLI_REPORTS
from .source import LoginDF, SearchQueries
from .xlsx_report import xlsx_report
from .xlsx_search_queries import xlsx_search_queries


def make_report(logger):
    df_report = LoginDF(logger)
    if df_report is None or df_report.shape[0] == 0:
        return None
    with pd.ExcelWriter("./report.xlsx", engine="xlsxwriter") as xlsx_writer:
        logger(logging.DEBUG, "BEGIN `report`", "app_debug")
        pd.formats.format.header_style["alignment"]["horizontal"] = "left"
        print("{:-<15}".format(""))
        print("\nReports:")
        map(lambda fn: fn(logger, xlsx_writer, df_report), xlsx_report)
        logger(logging.DEBUG, "END `report`", "app_debug")

    return df_report


def make_search_queries(logger):
    df_queries = SearchQueries(logger)
    if df_queries is None or df_queries.shape[0] == 0:
        return None
    with pd.ExcelWriter("./search_queries.xlsx", engine="xlsxwriter") as xlsx_writer:
        logger(logging.DEBUG, "BEGIN `search_queries`", "app_debug")
        pd.formats.format.header_style["alignment"]["horizontal"] = "left"
        print("{:-<15}".format(""))
        print("\nReports:")
        map(lambda fn: fn(logger, xlsx_writer, df_queries), xlsx_search_queries)
        logger(logging.DEBUG, "END `search_queries`", "app_debug")

    return df_queries


REPORTS = dict(zip(CLI_REPORTS, [make_report, make_search_queries]))
