# coding: utf-8
"""xlsx-reports"""
from .campaign_stat import (
    table_total_campaigns_stat,
    table_campaigns_type,
    plot_campaigns_type,
    table_list_campaigns_type
)
from .highlight import (
    table_highlight_type_phrases,
    plot_highlight_type_phrases,
    table_highlight_hist,
    plot_highlight_hist,
    table_highlight_type_campaign
)
from .words import (
    table_words_cnt,
    plot_words_cnt,
    table_top_words_usage,
    table_top_words_uppercase
)
from .templates import (
    table_templates_incorrect_usage,
    table_templates_status,
    plot_templates_status,
    table_display_url_has_template,
    table_wo_display_url
)
from .productivity import (
    table_productivity,
    plot_productivity,
    table_phrases_prod_lt,
)
from .img_shapes import (
    table_img_shapes,
    table_rsya_banners_wo_img,
)
from .timetarget import (
    table_avg_time_target,
    use_time_target_campaigns,
)
from .phrases import (  # noqa: F401
    table_top_nonbolded_phrases,
    chart_avg_position,
    table_top_phrases_lost_clicks,
    try_top_phrases_lost_clicks_wc,
    table_top_phrases_strange_pos,
    table_top_cpc_phrases
)
from .banners import (
    table_sitelinks_cnt,
    plot_sitelinks_cnt,
    table_bids_sl_count_lt4,
    table_description_sl_cnt,
    plot_description_sl_cnt,
    table_top_banners_zero_description,
    table_top_banners_navig_incorrect,
    table_callouts_cnt,
    plot_callouts_cnt,
    table_callouts_detail,
    plot_callouts_detail,
    table_top_banners_zero_callouts,
    table_top_banners_has_uppercase,
    table_top_banners_desktop_type_incorrect,
    table_top_banners_mobile_type_incorrect
)
from .group_stat import (
    table_group_stat_distribution,
)
from .vcard import (
    table_vcard_usage,
    plot_vcard_usage,
    table_banners_wo_vcard
)
from ..utils import note_maker as note, add_sheet_to_xlsx


def make_builder(fn, sheet_name, sheet_fmt=None, **kw_args):
    """((DataFrame) -> DataFrame, str, [(ExcelWriter, str, DataFrame) -> ExcelWriter], **kw_args) ->
       ((ExcelWriter, DataFrame) -> ExcelWriter)"""
    if sheet_fmt is None:
        sheet_fmt = []
    if not isinstance(sheet_fmt, list):
        raise TypeError("`sheet_fmt` must be iterable or None")

    def builder(logger, writer, df):
        add_sheet_to_xlsx(logger, writer, fn(df), sheet_name=sheet_name, sheet_fmt=sheet_fmt, **kw_args)

    return builder


# pd.option_context('display.max_colwidth', 200)

xlsx_report = [
    make_builder(
        table_total_campaigns_stat,
        "00_total_campaigns_stat",
        sheet_fmt=[note(u"Статистика по кампаниям.")],
        index=None),
    make_builder(
        table_highlight_type_phrases,
        "01_highlight_type",
        sheet_fmt=[
            plot_highlight_type_phrases,
            note(u"Подсветка - доля слов от рекламной фразы в объявлении: заголовок + тело баннера")]),
    make_builder(
        lambda df: table_highlight_hist(df, normalize=False),
        "01_highlight_hist",
        startcol=7),
    make_builder(
        lambda df: table_highlight_hist(df, normalize=True),
        "01_highlight_hist",
        sheet_fmt=[
            plot_highlight_hist,
            note(u"Подсветка - доля слов от рекламной фразы в объявлении: заголовок + тело баннера")]),
    make_builder(
        table_highlight_type_campaign,
        "01_campaign_highlight_type",
        sheet_fmt=[
            note(u"Подсветка - доля слов от рекламной фразы в объявлении: заголовок + тело баннера")]),
    make_builder(
        table_top_nonbolded_phrases,
        "01_top_non-bolded_phrases",
        sheet_fmt=[
            note(u"Подсветка - доля слов от рекламной фразы в объявлении: заголовок + тело баннера")],
        index=None),
    make_builder(
        table_sitelinks_cnt,
        "02_sitelinks_cnt",
        sheet_fmt=[
            plot_sitelinks_cnt,
            note(u"Использование быстрых ссылок для фраз с показами на поиске.")]),
    make_builder(
        table_bids_sl_count_lt4,
        "02_bids_sl_count_lt4",
        sheet_fmt=[
            note(u"Топ баннеров без максимального количества БС.")],
        index=None),
    make_builder(
        table_description_sl_cnt,
        "02_description_sl_cnt",
        sheet_fmt=[
            plot_description_sl_cnt,
            note(u"Использование описаний для быстрых ссылок по фразам с показами на поиске.")]),
    make_builder(
        table_top_banners_zero_description,
        "02_description_eq_zero",
        sheet_fmt=[
            note(u"Топ баннеров без описаний к БС.")],
        index=None),
    make_builder(
        table_top_banners_navig_incorrect,
        "02_navig_incorrect",
        sheet_fmt=[
            note(u"Топ баннеров (без фильтрации навигационных запросов) с максимальным количеством БС, но без описаний - обязательное условие для навигационного ответа.")],
        index=None),
    make_builder(
        table_callouts_cnt,
        "03_callouts_cnt",
        sheet_fmt=[
            plot_callouts_cnt,
            note(u"Использование уточнений по немобильным баннерам со статистикой на поиске (уточнения работают только по десктопном поиске).")]),
    make_builder(
        lambda df: table_callouts_detail(df, normalize=False),
        "03_callouts_detail",
        startcol=7),
    make_builder(
        lambda df: table_callouts_detail(df, normalize=True),
        "03_callouts_detail",
        sheet_fmt=[
            plot_callouts_detail,
            note(u"Использование уточнений по немобильным баннерам со статистикой на поиске (уточнения работают только по десктопном поиске).")]),
    make_builder(
        table_top_banners_zero_callouts,
        "03_top_banners_zero_callouts",
        sheet_fmt=[
            note(u"Топ баннеров без уточнений.")],
        index=None),
    make_builder(
        table_top_banners_has_uppercase,
        "03_top_banners_has_uppercase",
        sheet_fmt=[
            note(u"Топ баннеров со словами из ЗАГЛАВНЫХ.")],
        index=None),
    make_builder(
        table_campaigns_type,
        "04_campaigns_type",
        sheet_fmt=[
            plot_campaigns_type,
            note(u"Использование площадок. Факт использования выводится из фактической статистики, а не настроек.")]),
    make_builder(
        table_list_campaigns_type,
        "04_list_campaigns_type",
        sheet_fmt=[note(u"Список кампаний с типом используемых площадок.")],
        index=None),
    make_builder(
        lambda df: table_words_cnt(df, normalize=False),
        "05_words_cnt",
        sheet_fmt=[note(u"Использование рекламных фраз с показами на поиске по количеству слов.")],
        startcol=7),
    make_builder(
        lambda df: table_words_cnt(df, normalize=True),
        "05_words_cnt",
        sheet_fmt=[plot_words_cnt]),
    make_builder(
        table_templates_status,
        "06_templates_status",
        sheet_fmt=[
            plot_templates_status,
            note(u"Использование шаблонов в заголовке и теле баннеров с показами на поиске.")]),
    make_builder(
        table_templates_incorrect_usage,
        "06_templates_incorrect_usage",
        sheet_fmt=[note(u"Топ баннеров и тексто не проходящих проверку по максимальному кол-ву символов после подстановки рекламной фразы.")]),
    make_builder(
        table_display_url_has_template,
        "06_display_url",
        sheet_fmt=[
            plot_templates_status,
            note(u"Использование отображаемой ссылки в баннерах с показами на поиске.")]),
    make_builder(
        table_wo_display_url,
        "06_wo_display_url",
        sheet_fmt=[note(u"Топ баннеров без отображаемой ссылки.")]),
    make_builder(
        lambda df: table_productivity(df, normalize=False),
        "07_productivity",
        startcol=7,
        sheet_fmt=[note(u"Обзор продуктивности фраз.")]),
    make_builder(
        lambda df: table_productivity(df, normalize=True),
        "07_productivity",
        sheet_fmt=[plot_productivity]),
    make_builder(
        table_phrases_prod_lt,
        "07_phrases_prod_lt5",
        sheet_fmt=[note(u"Топ баннеров с продуктивностью меньше 5.")]),
    make_builder(
        table_img_shapes,
        "08_img_shapes",
        index=None,
        sheet_fmt=[note(u"Обзор используемых размеров изображений для баннеров с показами в сетях.")]),
    make_builder(
        table_rsya_banners_wo_img,
        "09_rsya_banners_wo_img",
        sheet_fmt=[note(u"Топ баннеров без картинок с показами в сетях")]),
    make_builder(
        table_avg_time_target,
        "10_avg_time_target",
        sheet_fmt=[note(u"Усредненные настройки временного таргетинга.")]),
    make_builder(
        use_time_target_campaigns,
        "10_use_time_target_campaigns",
        index=None,
        sheet_fmt=[note(u"Список кампаний с настроками временного таргетинга отличными от умолчательных.")]),
    make_builder(
        table_top_phrases_lost_clicks,
        "11_top_phrases_lost_clicks",
        sheet_fmt=[note(u"Топ фраз по упущенным кликам. Потенциал по принципу \"все показы в спеце\". Только для фраз с известным CTR в спеце.")]),
    make_builder(
        table_group_stat_distribution,
        "12_group_stat_distribution",
        sheet_fmt=[note(u"Распределение статистики по количеству фраз в группах объявлений.")]),
    make_builder(
        table_top_phrases_strange_pos,
        "12_top_phrases_strange_pos",
        index=None,
        sheet_fmt=[note(u"Топ \"странных\" фраз - небольшой разницей по стоимости между блоками и с показами в юж./правом")]),
    make_builder(
        table_top_cpc_phrases,
        "13_top_cpc_phrases",
        index=None,
        sheet_fmt=[note(u"Топ баннеро-фраз по стоимости клика.")]),
    make_builder(
        table_top_words_usage,
        "13_top_words_usage",
        sheet_fmt=[note(u"Топ используемых слов по количеству кликов на слово")]),
    make_builder(
        table_top_words_uppercase,
        "13_top_words_uppercase",
        sheet_fmt=[note(u"Топ прописных слов из текстов объявлений по количеству кликов на слово")]),
    make_builder(
        table_vcard_usage,
        "14_vcard_usage",
        sheet_fmt=[
            plot_vcard_usage,
            note(
                u'Использование визиток для баннеров с показами на поиске.\n' +
                u'Доп. поля:\n"Street",\nany("House" | "Building" | "Apartment")\n"ExtraMessage"\n' +
                u'"ContactEmail",\n"Ogrn",\n"MetroStationId"', width=200, height=180)]),
    make_builder(
        table_banners_wo_vcard,
        "14_banners_wo_vcard",
        sheet_fmt=[note(u"Топ баннеров без визиток с показами на поиске.")]),
    make_builder(
        table_top_banners_desktop_type_incorrect,
        "15_banners_desk_w_mob_stat",
        index=None,
        sheet_fmt=[note(u"Топ десктопных баннеров с мобильными показами.")]),
    make_builder(
        table_top_banners_mobile_type_incorrect,
        "15_banners_mob_w_desk_stat",
        index=None,
        sheet_fmt=[note(u"Топ мобильных баннеров с десктопными показами.")]),
]
