# coding: utf-8
"""xlsx-report:
  - 02_sitelinks_cnt
  - 02_bids_sl_count_lt4
  - 02_description_sl_cnt
  - 02_description_eq_zero
  - 02_navig_incorrect
  - 03_callouts_cnt
  - 03_callouts_detail
  - 03_top_banners_has_uppercase
  - 03_top_banners_zero_callouts
  - 15_banners_desk_w_mob_stat
  - 15_banners_mob_w_desk_stat


Help:
[Quick Links](https://yandex.ru/support/direct/features/quick-links.xml) (Search)
[Callouts](https://yandex.ru/support/direct/features/callout.xml?lang=ru) (Search)
[Описания](https://yandex.ru/support/direct-news/n-2015-10-26.xml) (Search)
"""
# import seaborn as sns
from ..const import EXCEL_MAX_ROWS
from ..xlsx_chart.chart import Chart, chart_add_series, FONT_NAME
from ..utils import (
    search_cid,
    search_bid,
    has_uppercase_word,
    filter_by_adtype
)
import pandas as pd


def table_sitelinks_cnt(df):
    """02_sitelinks_cnt"""

    df_chart = filter_by_adtype(df, "TEXT_AD")
    df_chart = (df_chart[(df_chart.ShowsSearch > 0)]
                [["ShowsSearch", "ClicksSearch", "CostSearch", "SitelinksCnt", "DirectBannerID"]]
                .groupby("SitelinksCnt")
                .agg({"ShowsSearch": pd.np.sum,
                      "ClicksSearch": pd.np.sum,
                      "CostSearch": pd.np.sum,
                      "DirectBannerID": lambda x: x.unique().size}))
    if df_chart.shape[0] == 0:
        return None
    max_sl_cnt = 4  # Теоритический максимум по sitelinks (2016-09))
    df_chart.rename(columns={
        "DirectBannerID": u"Кол-во баннеров",
        "ShowsSearch": u"Показы (поиск)",
        "ClicksSearch": u"Клики (поиск)",
        "CostSearch": u"Затраты (поиск)"},
        inplace=True)
    df_chart.index.name = u"Кол-во быстрых ссылок"
    idx = range(max_sl_cnt + 1)

    return df_chart.loc[idx].fillna("")


def plot_sitelinks_cnt(writer, sheet_name, df):
    """(ExcelWriter, str, DataFrame) -> ExcelWriter"""

    sheet = writer.sheets[sheet_name]

    chart = Chart(writer.book, "column", "percent_stacked", u'Использование быстрых ссылок')
    positions = ("inside_end", "center")
    series = range(5)
    for x in series:
        irow = 2 + x  # 2 is offset to values
        pos_type = positions[x == series[0] or x == series[-1]]
        chart_add_series(chart, x, {
            'values': '={0}!B{1}:E{1}'.format(sheet_name, irow),
            "categories": '={}!B1:E1'.format(sheet_name),
            "name": "={0}!A{1}".format(sheet_name, irow),
            "data_labels": {"position": pos_type}})

    sheet.insert_chart('A8', chart)

    return writer


def table_bids_sl_count_lt4(df):
    """02_bids_sl_count_lt4"""

    max_sl_cnt = 4
    df_chart = filter_by_adtype(df, "TEXT_AD")
    df_chart = (df_chart[(df_chart.SitelinksCnt < max_sl_cnt)]
                [["ShowsSearch", "ClicksSearch", "CostSearch", "CampaignID", "DirectBannerID", "SitelinksCnt"]]
                .groupby(['CampaignID', 'DirectBannerID', 'SitelinksCnt'], as_index=False)
                .agg({"ShowsSearch": pd.np.sum,
                      "ClicksSearch": pd.np.sum,
                      "CostSearch": pd.np.sum})
                .sort_values('ShowsSearch', ascending=False)
                .head(EXCEL_MAX_ROWS))
    if df_chart.shape[0] == 0:
        return None
    df_chart['HelperUrlCid'] = df_chart['CampaignID'].apply(search_cid)
    df_chart['HelperUrlBid'] = df_chart['DirectBannerID'].apply(search_bid)
    df_chart.rename(columns={
        "CampaignID": u"Кампания №",
        "DirectBannerID": u"Баннер №",
        "SitelinksCnt": u"Кол-во быстрых ссылок",
        "ShowsSearch": u"Показы (поиск)",
        "ClicksSearch": u"Клики (поиск)",
        "CostSearch": u"Затраты (поиск)",
        "HelperUrlCid": u"Ссылка на кампанию",
        "HelperUrlBid": u"Ссылка на баннер"},
        inplace=True)

    return df_chart


def table_description_sl_cnt(df):
    """02_description_sl_cnt"""

    df_chart = filter_by_adtype(df, "TEXT_AD")
    df_chart = (df_chart[(df_chart.ShowsSearch > 0)]
                [["ShowsSearch", "ClicksSearch", "CostSearch", "DescriptionSLCnt", "DirectBannerID"]]
                .groupby("DescriptionSLCnt")
                .agg({"ShowsSearch": pd.np.sum,
                      "ClicksSearch": pd.np.sum,
                      "CostSearch": pd.np.sum,
                      "DirectBannerID": lambda x: x.unique().size}))
    if df_chart.shape[0] == 0:
        return None
    max_sl_cnt = 4  # Теоритический максимум по sitelinks (2016-09))
    df_chart.rename(columns={
        "DirectBannerID": u"Кол-во баннеров",
        "ShowsSearch": u"Показы (поиск)",
        "ClicksSearch": u"Клики (поиск)",
        "CostSearch": u"Затраты (поиск)"},
        inplace=True)
    df_chart.index.name = u"Кол-во описаний к БС"

    return df_chart.loc[range(max_sl_cnt + 1)].fillna("")


def plot_description_sl_cnt(writer, sheet_name, df):
    """(ExcelWriter, str, DataFrame) -> ExcelWriter"""

    sheet = writer.sheets[sheet_name]

    chart = Chart(writer.book, "column", "percent_stacked", u'Использование описаний к БС')
    positions = ("inside_end", "center")
    series = range(5)
    for x in series:
        irow = 2 + x  # 2 is offset to values
        pos_type = positions[x == series[0] or x == series[-1]]
        chart_add_series(chart, x, {
            'values': '={0}!B{1}:E{1}'.format(sheet_name, irow),
            "categories": '={}!B1:E1'.format(sheet_name),
            "name": "={0}!A{1}".format(sheet_name, irow),
            "data_labels": {"position": pos_type}})

    sheet.insert_chart('A8', chart)

    return writer


def table_top_banners_zero_description(df):
    """02_description_eq_zero"""
    df_chart = filter_by_adtype(df, "TEXT_AD")
    df_chart = (df_chart[(df_chart.DescriptionSLCnt == 0) & (df_chart.ShowsSearch > 0)]
                .groupby(["CampaignID", "DirectBannerID"], as_index=False)
                .agg({"ShowsSearch": pd.np.sum,
                      "ClicksSearch": pd.np.sum,
                      "CostSearch": pd.np.sum})
                .sort_values("ShowsSearch", ascending=False)
                .reset_index(drop=True)
                .head(EXCEL_MAX_ROWS)
                .assign(HelperUrlCid=lambda x: x.CampaignID.apply(search_cid),
                        HelperUrlBid=lambda x: x.DirectBannerID.apply(search_bid)))
    if df_chart.shape[0] == 0:
        return None
    df_chart.rename(columns={
        "CampaignID": u"Кампания №",
        "DirectBannerID": u"Баннер №",
        "ShowsSearch": u"Показы (поиск)",
        "ClicksSearch": u"Клики (поиск)",
        "CostSearch": u"Затраты (поиск)",
        "HelperUrlCid": u"Ссылка на кампанию",
        "HelperUrlBid": u"Ссылка на баннер"},
        inplace=True)

    return df_chart


def table_top_banners_navig_incorrect(df):
    """02_navig_incorrect
    Navig-answer is impossible when SL equal MAX_SL and Descriptions less then MAX_SL
    """
    MAX_SL = 4
    df_chart = filter_by_adtype(df, "TEXT_AD")
    df_chart = (df_chart[
        (df_chart.DescriptionSLCnt < MAX_SL) &
        (df_chart.SitelinksCnt == MAX_SL) &
        (df_chart.ShowsSearch > 0)
    ][["CampaignID", "DirectBannerID", "PhraseText",
       "ShowsSearch", "ClicksSearch", "CostSearch"]]
        .groupby(["CampaignID", "DirectBannerID", "PhraseText"], as_index=False)
        .agg({"ShowsSearch": pd.np.sum,
              "ClicksSearch": pd.np.sum,
              "CostSearch": pd.np.sum})
        .sort_values("ShowsSearch", ascending=False)
        .reset_index(drop=True)
        .head(EXCEL_MAX_ROWS))
    if df_chart.shape[0] == 0:
        return None
    df_chart.rename(columns={
        "CampaignID": u"Кампания №",
        "DirectBannerID": u"Баннер №",
        "PhraseText": u"Рекламная фраза",
        "ShowsSearch": u"Показы (поиск)",
        "ClicksSearch": u"Клики (поиск)",
        "CostSearch": u"Затраты (поиск)"},
        inplace=True)

    return df_chart[[u"Кампания №", u"Баннер №", u"Рекламная фраза", u"Показы (поиск)", u"Клики (поиск)", u"Затраты (поиск)"]]


def table_callouts_cnt(df):
    """03_callouts_cnt"""

    df_chart = filter_by_adtype(df, "TEXT_AD")
    df_chart = df_chart[
        (df_chart.ShowsSearch > 0) &
        (df_chart.isMobileBanner == False)].copy()  # noqa:E712
    df_chart = (df_chart[["ShowsSearch", "ClicksSearch", "CostSearch", "CalloutsCnt", "DirectBannerID"]]
                .assign(CalloutsCnt=lambda df: df.CalloutsCnt > 0)
                .groupby("CalloutsCnt")
                .agg({"ShowsSearch": pd.np.sum,
                      "ClicksSearch": pd.np.sum,
                      "CostSearch": pd.np.sum,
                      "DirectBannerID": lambda x: x.unique().size}))
    if df_chart.shape[0] == 0:
        return None
    df_chart.rename(columns={
        "DirectBannerID": u"Кол-во баннеров",
        "ShowsSearch": u"Показы (поиск)",
        "ClicksSearch": u"Клики (поиск)",
        "CostSearch": u"Затраты (поиск)"},
        inplace=True)
    df_chart.rename(index={
        0: u"Без уточнений",
        1: u"Есть уточнения"},
        inplace=True)
    df_chart.index.name = u"Уточнения"

    return df_chart.loc[[u"Без уточнений", u"Есть уточнения"]].fillna("")


def plot_callouts_cnt(writer, sheet_name, df):
    """(ExcelWriter, str, DataFrame) -> ExcelWriter"""
    sheet = writer.sheets[sheet_name]
    chart = Chart(writer.book, "bar", "percent_stacked", u'Использование уточнений')
    series = range(2)
    for x in series:
        irow = 2 + x  # 2 is offset to values
        chart_add_series(chart, x, {
            'values': '={0}!B{1}:E{1}'.format(sheet_name, irow),
            "categories": '={}!B1:E1'.format(sheet_name),
            "name": "={0}!A{1}".format(sheet_name, irow)})

    sheet.insert_chart('A5', chart)

    return writer


def table_callouts_detail(df, normalize):
    """03_callouts_detail"""

    df_chart = filter_by_adtype(df, "TEXT_AD")
    df_chart = df_chart[
        (df_chart.ShowsSearch > 0) &
        (df_chart.isMobileBanner == False)
        ].copy()  # noqa:E712
    if df_chart.shape[0] == 0:
        return None
    df_chart = (df_chart[["ShowsSearch", "ClicksSearch", "CostSearch", "CalloutsCnt", "DirectBannerID"]]
                .groupby("CalloutsCnt")
                .agg({"ShowsSearch": pd.np.sum,
                      "ClicksSearch": pd.np.sum,
                      "CostSearch": pd.np.sum,
                      "DirectBannerID": lambda x: x.unique().size}))

    df_chart.rename(columns={
        "DirectBannerID": u"Кол-во баннеров",
        "ShowsSearch": u"Показы (поиск)",
        "ClicksSearch": u"Клики (поиск)",
        "CostSearch": u"Затраты (поиск)"},
        inplace=True)
    df_chart.index.name = u"Кол-во уточнений"

    if normalize:
        df_chart = df_chart.apply(lambda x: x / x.sum())

    return df_chart[[u"Кол-во баннеров", u"Показы (поиск)", u"Клики (поиск)", u"Затраты (поиск)"]]


def plot_callouts_detail(writer, sheet_name, df):
    """
    (ExcelWriter, str, DataFrame) -> ExcelWriter
    03_callouts_detail
    """

    sheet = writer.sheets[sheet_name]
    chart = Chart(writer.book, "column", None, u'Использование уточнений')
    chart.set_y_axis({"name": u"Доля от общего",
                      'name_font': {"size": 16, "bold": False, "name": FONT_NAME},
                      'num_font': {"size": 14, "bold": False, "name": FONT_NAME},
                      'num_format': '#0%',
                      'major_tick_mark': 'none',
                      'minor_tick_mark': 'none',
                      'line': {'none': True}})
    cols = "BCDE"
    last_row = df.index.size + 1
    for n, col in enumerate(cols):
        chart_add_series(chart, n, {
            'values': '={0}!{1}2:{1}{2}'.format(sheet_name, col, last_row),
            "categories": '={}!A2:A{}'.format(sheet_name, last_row),
            "name": "={0}!{1}1".format(sheet_name, col),
            "data_labels": {"value": False}})

    sheet.insert_chart('A%s' % (last_row + 2), chart)

    return writer


def table_top_banners_zero_callouts(df):
    """03_top_banners_zero_callouts"""

    df_chart = filter_by_adtype(df, "TEXT_AD")
    df_chart = df_chart[
        (df_chart.ShowsSearch > 0) &
        (df_chart.isMobileBanner == False)
        ].copy()  # noqa:E712
    df_chart = (df_chart[df_chart.CalloutsCnt == 0]
                .groupby(["CampaignID", "DirectBannerID"], as_index=False)
                .agg({"ShowsSearch": pd.np.sum,
                      "ClicksSearch": pd.np.sum,
                      "CostSearch": pd.np.sum})
                .sort_values("ShowsSearch", ascending=False)
                .reset_index(drop=True)
                .head(EXCEL_MAX_ROWS)
                .assign(HelperUrlCid=lambda x: x.CampaignID.apply(search_cid),
                        HelperUrlBid=lambda x: x.DirectBannerID.apply(search_bid)))
    if df_chart.shape[0] == 0:
        return None
    df_chart.rename(columns={
        "CampaignID": u"Кампания №",
        "DirectBannerID": u"Баннер №",
        "ShowsSearch": u"Показы (поиск)",
        "ClicksSearch": u"Клики (поиск)",
        "CostSearch": u"Затраты (поиск)",
        "HelperUrlCid": u"Ссылка на кампанию",
        "HelperUrlBid": u"Ссылка на баннер"},
        inplace=True)

    return df_chart


def table_top_banners_has_uppercase(df):
    """03_top_banners_has_uppercase"""

    df_chart = filter_by_adtype(df, "TEXT_AD")
    df_chart = df_chart[(df_chart["Shows"] > 0)][
        ["DirectBannerID", "Title", "Body", "Shows", "Clicks", "Cost"]].copy()
    if df_chart.shape[0] == 0:
        return None
    has_uppercase = (df_chart["Title"].apply(has_uppercase_word) | df_chart["Body"].apply(has_uppercase_word))
    df_chart = (df_chart[has_uppercase]
                .groupby(["DirectBannerID", "Title", "Body"], as_index=False)
                .agg({
                    "Shows": pd.np.sum,
                    "Clicks": pd.np.sum,
                    "Cost": pd.np.sum})
                .sort_values("Shows", ascending=False)
                .head(EXCEL_MAX_ROWS))

    if df_chart.shape[0] == 0:
        return None
    else:
        return df_chart


def table_top_banners_desktop_type_incorrect(df):
    """15_banners_desk_w_mob_stat"""

    df_chart = filter_by_adtype(df, "TEXT_AD")
    df_chart = (df_chart[(df_chart.isMobileBanner == False) & (df_chart.ShowsMobile > 0)]  # noqa:712
                .groupby(["CampaignID", "DirectBannerID", "Title", "Body"], as_index=False)
                .agg({"ShowsMobile": pd.np.sum,
                      "ClicksMobile": pd.np.sum,
                      "CostMobile": pd.np.sum})
                .sort_values("ShowsMobile", ascending=False)
                .reset_index(drop=True)
                .head(EXCEL_MAX_ROWS))
    if df_chart.shape[0] == 0:
        return None
    df_chart.rename(columns={
        "CampaignID": u"Кампания №",
        "DirectBannerID": u"Баннер №",
        "Title": u"Заголовок",
        "Body": u"Объявление",
        "ShowsMobile": u"Показы (моб.)",
        "ClicksMobile": u"Клики (моб.)",
        "CostMobile": u"Затраты (моб.)"},
        inplace=True)

    return df_chart


def table_top_banners_mobile_type_incorrect(df):
    """15_banners_mob_w_desk_stat"""

    df_chart = filter_by_adtype(df, "TEXT_AD")
    df_chart = (df_chart[df_chart.isMobileBanner & (df_chart.ShowsDesktop > 0)]  # noqa:712
                .groupby(["CampaignID", "DirectBannerID", "Title", "Body"], as_index=False)
                .agg({"ShowsDesktop": pd.np.sum,
                      "ClicksDesktop": pd.np.sum,
                      "CostDesktop": pd.np.sum})
                .sort_values("ShowsDesktop", ascending=False)
                .reset_index(drop=True)
                .head(EXCEL_MAX_ROWS))
    if df_chart.shape[0] == 0:
        return None
    df_chart.rename(columns={
        "CampaignID": u"Кампания №",
        "DirectBannerID": u"Баннер №",
        "Title": u"Заголовок",
        "Body": u"Объявление",
        "ShowsDesktop": u"Показы (не моб.)",
        "ClicksDesktop": u"Клики (не моб.)",
        "CostDesktop": u"Затраты (не моб.)"},
        inplace=True)

    return df_chart
