# coding: utf-8
"""xlsx-report:
  - 08_img_shapes
  - 09_rsya_banners_wo_img
"""
from ..const import EXCEL_MAX_ROWS, IMG_MAX_DOWNLOAD
from ..utils import get_image_shape
from os.path import isfile
from multiprocessing.dummy import Pool
import pandas as pd
import seaborn as sns


def fetch_img_shapes(df):
    """(DataFrame) -> DataFrame?
    08_img_shapes
    """
    if not isfile("./tmp_data/img_shapes.pkl"):
        img_shapes = df[df.ImageHash != 'None'].ImageHash.unique()
        if 0 < img_shapes.size <= IMG_MAX_DOWNLOAD:
            print("...%s images are downloading" % img_shapes.size)
            img_shapes = Pool(4).map(get_image_shape, img_shapes, 10)
            img_shapes = pd.DataFrame([{
                "ImageHash": k[0],
                "height": k[1],
                "width": k[2]
            } for k in img_shapes])
            img_shapes.to_pickle("./tmp_data/img_shapes.pkl")
            print("...dowload %s images" % img_shapes.shape[0])
        else:
            print(
                "too many images or none: %s. Current `IMG_MAX_DOWNLOAD` is: %s"
                % (img_shapes.size, IMG_MAX_DOWNLOAD))
            img_shapes = None
    else:
        img_shapes = pd.read_pickle("./tmp_data/img_shapes.pkl")
        print("...load %s images from ./tmp_data/img_shapes.pkl" %
              img_shapes.shape[0])

    return img_shapes


def chart_table_img_shapes(df, img_shapes):
    """
    (DataFrame, DataFrame?) -> DataFrame?
    08_img_shapes
    """

    if img_shapes is None:
        return None

    df_chart = df[[
        "ShowsRSYA", "ClicksRSYA", "CostRSYA", "ImageHash", "DirectBannerID"
    ]].copy()
    df_chart = pd.merge(df_chart, img_shapes.drop_duplicates(), on="ImageHash")
    df_chart = (df_chart[df_chart.ShowsRSYA > 0].groupby(
        ["ImageHash", "height", "width", "DirectBannerID"], as_index=False)
                .agg({
                    "ShowsRSYA": pd.np.sum,
                    "ClicksRSYA": pd.np.sum,
                    "CostRSYA": pd.np.sum
                }))
    df_chart.sort_values("ShowsRSYA", ascending=False, inplace=True)

    if df_chart.shape[0] > 1:
        sns.plt.figure()
        sns.jointplot(df_chart.width, df_chart.height, kind="scatter")
        sns.plt.savefig("./charts/08_img_shapes.png", dpi=200)

    return df_chart


def table_img_shapes(df):
    """08_img_shapes"""
    img_shapes = fetch_img_shapes(df)
    df_chart = chart_table_img_shapes(df, img_shapes)
    if df_chart is None:
        return None

    return (df_chart[df_chart.ShowsRSYA > 0]
            .groupby(["height", "width"], as_index=False).agg({
                "ShowsRSYA":
                pd.np.sum,
                "ClicksRSYA":
                pd.np.sum,
                "CostRSYA":
                pd.np.sum,
                "DirectBannerID":
                lambda x: x.unique().size,
                "ImageHash":
                lambda x: x.unique().size
            }).rename(columns={
                "ImageHash": "UniqImgCnt",
                "DirectBannerID": "BannersCnt"
            }).assign(area=lambda x: x.height * x.width).sort_values("area")
            .reset_index(drop=True).drop("area", axis=1).rename(columns={
                "height":
                u"Высота",
                "width":
                u"Ширина",
                "ShowsRSYA":
                u"Показы (сети)",
                "ClicksRSYA":
                u"Клики (сети)",
                "CostRSYA":
                u"Затраты (сети)",
                "UniqImgCnt":
                u"Кол-во разных картинок",
                "BannersCnt":
                u"Кол-во баннеров"
            }))


def table_rsya_banners_wo_img(df):
    """09_rsya_banners_wo_img"""

    df_chart = (df[(df.ImageHash == 'None') & (df.ShowsRSYA > 0)][[
        "CampaignID", "DirectBannerID", "Title", "Body", "ShowsRSYA",
        "ClicksRSYA", "CostRSYA"
    ]].groupby(["CampaignID", "DirectBannerID", "Title", "Body"]).agg({
        "ShowsRSYA":
        pd.np.sum,
        "ClicksRSYA":
        pd.np.sum,
        "CostRSYA":
        pd.np.sum
    }).sort_values("ShowsRSYA", ascending=False).head(EXCEL_MAX_ROWS))
    df_chart.rename(
        columns={
            "ShowsRSYA": u"Показы (сети)",
            "ClicksRSYA": u"Клики (сети)",
            "CostRSYA": u"Затраты (сети)"
        },
        inplace=True)
    df_chart.index.names = [
        u"Кампания №", u"Баннер №", u"Заголовок", u"Объявление"
    ]
    if df_chart.shape[0] == 0:
        return None

    return df_chart
