# coding: utf-8
"""xlsx-reports:
  - 01_campaigns_highlight_type
  - 01_top_non-bolded_phrases
  - 11_top_phrases_lost_clicks
  - 11_top_phrases_lost_clicks_wc
  - 12_top_phrases_strange_pos
  - 13_top_cpc_phrases
"""
from ..const import EXCEL_MAX_ROWS, __HAS_WC__
from ..utils import (
    search_bid,
    phrase_process,
    filter_by_adtype
)
import seaborn as sns
import pandas as pd
if __HAS_WC__:
    import wordcloud


def table_top_nonbolded_phrases(df):
    """01_top_non-bolded_phrases"""

    df_chart = filter_by_adtype(df, "TEXT_AD")
    df_chart = df_chart[[
        "BoldedWordsCnt",
        "WordsCnt",
        "ShowsSearch",
        "ClicksSearch",
        "CostSearch",
        "PhraseText",
        "NonBoldedWords",
        "Title",
        "Body",
        "DirectBannerID"]].copy()
    df_chart = df_chart[(df_chart.WordsCnt != 0) & (df_chart.ShowsSearch > 0)]
    df_chart["PhraseBoldShare"] = df_chart.BoldedWordsCnt / df_chart.WordsCnt
    df_chart.drop(["BoldedWordsCnt", "WordsCnt"], axis=1, inplace=True)
    df_chart['ShowsSearchNonBolded'] = ((1 - df_chart['PhraseBoldShare']) * df_chart['ShowsSearch']).astype(int)
    df_chart.sort_values("ShowsSearchNonBolded", ascending=False, inplace=True)
    df_chart["HelperURL"] = df_chart.DirectBannerID.apply(lambda x: search_bid(x))
    df_chart = df_chart[["Title", "Body", "PhraseText", "NonBoldedWords",
                         "ShowsSearch", "ClicksSearch", "CostSearch", "HelperURL"]]
    df_chart.rename(columns={
        "Title": u"Заголовок",
        "Body": u"Объявление",
        "PhraseText": u"Рекламная фраза",
        "NonBoldedWords": u"Неподсвеченное слово",
        "ShowsSearch": u"Показы (поиск)",
        "ClicksSearch": u"Клики (поиск)",
        "CostSearch": u"Затраты (поиск)",
        "HelperURL": u"Ссылка на баннер"},
        inplace=True)

    return df_chart


def chart_avg_position(df):
    """
    avg_position
    non-used
    """
    if "None" in df.AvgPosShowsSpec.unique():
        pred = (df.AvgPosShowsSpec != "None") & (df.ShowsSpec > 50)
    else:
        pred = df.ShowsSpec > 50

    df_chart = df[pred].copy()
    df_chart.AvgPosShowsSpec = df_chart.AvgPosShowsSpec.astype(float)
    df_chart["CTR_spec"] = df_chart.ClicksSpec / df_chart.ShowsSpec
    if df_chart.shape[0] > 0:
        sns.jointplot(df_chart.AvgPosShowsSpec, df_chart.CTR_spec)


def table_top_phrases_lost_clicks(df):
    """11_top_phrases_lost_clicks"""

    df_chart = filter_by_adtype(df, "TEXT_AD")
    df_chart = (df_chart[
        (df_chart.ClicksSpec > 0) &
        (df_chart.ClicksSpec <= df_chart.ShowsSpec)
        ]
        .assign(PhraseText=lambda x: x.PhraseText.apply(phrase_process))
        [["PhraseText", "ShowsSpec", "ClicksSpec", "ShowsOther", "ClicksOther"]]
        .groupby("PhraseText")
        .agg({"ShowsSpec": pd.np.sum,
              "ClicksSpec": pd.np.sum,
              "ShowsOther": pd.np.sum,
              "ClicksOther": pd.np.sum})
        .assign(MaxClicks=lambda x: x.ClicksSpec / x.ShowsSpec * (x.ShowsOther + x.ShowsSpec))
        .assign(LostClicks=lambda x: x.ClicksSpec / x.ShowsSpec * x.ShowsOther - x.ClicksOther)
        .sort_values("LostClicks", ascending=False)
        .head(EXCEL_MAX_ROWS))
    if df_chart.shape[0] == 0:
        return None
    if __HAS_WC__:
        try_top_phrases_lost_clicks_wc(df_chart)

    df_chart.rename(columns={
        "ShowsSpec": u"Показы (спец.)",
        "ClicksSpec": u"Клики (спец.)",
        "CostSpec": u"Затраты (спец.)",
        "ShowsOther": u"Показы (юж./пр. блок)",
        "ClicksOther": u"Клики (юж./пр. блок)",
        "CostOther": u"Затраты (юж./пр. блок)",
        "MaxClicks": u"Максимум кликов",
        "LostClicks": u"Упущенные клики"},
        inplace=True)
    df_chart.index.names = [u"Рекламная фраза"]

    return df_chart


def try_top_phrases_lost_clicks_wc(df_chart):
    """11_top_phrases_lost_clicks_wc"""
    if df_chart.shape[0] > 0:
        MAX_WORDS = 250  # noqa: N806 (lowercase)
        wc = wordcloud.WordCloud(width=1000,
                                 max_words=MAX_WORDS,
                                 # font_path='/home/n-bar/.fonts/Textbook_New/TBN55.otf',
                                 height=1000,
                                 # relative_scaling=.6,
                                 background_color=(150, 150, 150, 20),
                                 random_state=42,
                                 mode="RGBA")

        word_cloud = wc.generate_from_frequencies([(k, int(v)) for k, v in df_chart[:MAX_WORDS].LostClicks.to_dict().items()])
        word_cloud.to_file('./charts/11_top_phrases_lost_clicks.png')
        print("11_top_phrases_lost_clicks_wc")
        word_cloud.to_image()
    else:
        print("11_top_phrases_lost_clicks_wc is empty")


def table_top_phrases_strange_pos(df):
    """12_top_phrases_strange_pos"""

    df_chart = df[df.AvgPosShowsOther != "None"].copy()
    df_chart.AvgPosShowsOther = df_chart.AvgPosShowsOther.astype(float)
    df_chart["ShowsSpecShare"] = df_chart["ShowsSpec"] / df_chart["ShowsSearch"]
    df_chart["CPC_spec"] = df_chart["CostSpec"] / df_chart["ClicksSpec"]
    df_chart["CPC_other"] = df_chart["CostOther"] / df_chart["ClicksOther"]
    df_chart["CPC_ratio"] = df_chart["CPC_spec"] / df_chart["CPC_other"]
    df_chart["CTR_spec"] = df_chart["ClicksSpec"] / df_chart["ShowsSpec"]
    df_chart["CTR_other"] = df_chart["ClicksOther"] / df_chart["ShowsOther"]
    df_chart = (df_chart[(df_chart["ShowsSpecShare"] < .9) &
                         (df_chart["CPC_ratio"] < 2) &
                         (df_chart.AvgPosShowsOther < 4.5) &
                         (df_chart.ShowsOther > 50)]
                [["DirectBannerID", "PhraseText", "AvgPosShowsOther", "ShowsOther", "ShowsSpec",
                  "CPC_spec", "CPC_other", "CTR_spec", "CTR_other"]]
                .sort_values("ShowsOther", ascending=False)
                .reset_index(drop=True)
                .fillna(0)
                .round(3))
    if df_chart.shape[0] == 0:
        return None
    df_chart.rename(columns={
        "DirectBannerID": u"Баннер №",
        "PhraseText": u"Рекламная фраза",
        "AvgPosShowsOther": u"Ср. поз. показов (юж./пр. блок)",
        "ShowsOther": u"Показы (юж./пр. блок)",
        "ShowsSpec": u"Показы (спец)",
        "CPC_other": u"CPC (юж./пр. блок)",
        "CPC_spec": u"CPC (спец)",
        "CTR_other": u"CTR (юж./пр. блок)",
        "CTR_spec": u"CTR (спец.)"},
        inplace=True)

    return df_chart


def table_top_cpc_phrases(df):
    """13_top_cpc_phrases"""

    df_chart = (df[["DirectBannerID", "PhraseText", "Shows", "Clicks", "Cost"]]
                .groupby(["DirectBannerID", "PhraseText"], as_index=False)
                .sum()
                .assign(CPC=lambda x: x.Cost / x.Clicks)
                .fillna(0)
                .sort_values("CPC", ascending=False)
                .reset_index(drop=True)
                .drop("Cost", axis=1)
                .head(EXCEL_MAX_ROWS))
    if df_chart.shape[0] == 0:
        return None
    df_chart.rename(columns={
        "DirectBannerID": u"Баннер №",
        "PhraseText": u"Рекламная фраза",
        "AvgPosShowsOther": u"Ср. поз. показов (юж./пр. блок)",
        "Shows": u"Показы",
        "Clicks": u"Клики",
        "CPC": u"CPC"},
        inplace=True)

    return df_chart
