# coding: utf-8
"""xlsx-report:
  - 07_productivity
  - 07_phrases_prod_lt5

Help:
[Productivity]
  (https://yandex.ru/support/direct/efficiency/productivity.xml?lang=en)
"""
from ..const import EXCEL_MAX_ROWS
from ..xlsx_chart.chart import Chart, chart_add_series, FONT_NAME
import pandas as pd


def table_productivity(df, normalize):
    """07_productivity"""

    df_chart = (df[[
        "Shows", "Clicks", "Cost", "Productivity", "BSPhraseID",
        "DirectBannerID"
    ]].groupby(
        ["Productivity", "BSPhraseID", "DirectBannerID"], as_index=False).agg({
            "Shows":
            pd.np.sum,
            "Clicks":
            pd.np.sum,
            "Cost":
            pd.np.sum
        }))

    if "None" in df_chart.Productivity.unique():
        df_chart = df_chart[df_chart.Productivity != "None"]

    if df_chart.shape[0] == 0:
        return None

    df_chart.Productivity = df_chart.Productivity.apply(
        lambda x: round(float(x)))
    df_chart["PhraseCnt"] = 1
    df_chart = (df_chart.groupby("Productivity").agg({
        "Shows": pd.np.sum,
        "Clicks": pd.np.sum,
        "Cost": pd.np.sum,
        "PhraseCnt": pd.np.sum
    }))
    df_chart.rename(
        columns={
            "PhraseCnt": u"Кол-во фраз",
            "Shows": u"Показы",
            "Clicks": u"Клики",
            "Cost": u"Затраты"
        },
        inplace=True)
    df_chart.index.names = [u"Продуктивность"]
    df_chart = df_chart.loc[range(11)].fillna(pd.np.nan)

    if normalize:
        df_chart = df_chart.apply(lambda x: x / x.sum())

    return df_chart


def plot_productivity(writer, sheet_name, df):
    """(ExcelWriter, str, DataFrame) -> ExcelWriter"""

    sheet = writer.sheets[sheet_name]
    chart = Chart(writer.book, "column", None, u'Продуктивность фраз')
    chart.set_y_axis({
        "name": u"Доля от общего",
        'name_font': {
            "size": 16,
            "bold": False,
            "name": FONT_NAME
        },
        'num_font': {
            "size": 14,
            "bold": False,
            "name": FONT_NAME
        },
        'num_format': '#0%',
        'major_tick_mark': 'none',
        'minor_tick_mark': 'none',
        'line': {
            'none': True
        }
    })
    cols = "BCDE"
    last_row = df.index.size + 1
    for n, col in enumerate(cols):
        chart_add_series(chart, n, {
            'values':
            '={0}!{1}2:{1}{2}'.format(sheet_name, col, last_row),
            "categories":
            '={}!A2:A{}'.format(sheet_name, last_row),
            "name":
            "={0}!{1}1".format(sheet_name, col),
            "data_labels": {
                "value": False
            }
        })

    sheet.insert_chart('A%s' % (last_row + 2), chart)

    return writer


def table_phrases_prod_lt(df, n=5):
    """07_phrases_prod_lt5"""

    if "None" in df.Productivity.unique():
        df_chart = df.loc[df.Productivity != "None"].copy()
    else:
        df_chart = df.copy()

    df_chart.Productivity = df_chart.Productivity.apply(
        lambda x: round(float(x)))
    df_chart = (df_chart.loc[(df_chart.Productivity < n) &
                             (df_chart.ShowsSearch > 0)]
                .sort_values("Shows", ascending=False).head(EXCEL_MAX_ROWS)
                .set_index("DirectBannerID")[[
                    "Title", "Body", "PhraseText", "Productivity",
                    "ShowsSearch", "ClicksSearch", "CostSearch"
                ]])
    if df_chart.shape[0] == 0:
        return None
    df_chart.rename(
        columns={
            "Title": u"Заголовок",
            "Body": u"Объявление",
            "PhraseText": u"Рекламная фраза",
            "Productivity": u"Продуктивность",
            "ShowsSearch": u"Показы (поиск)",
            "ClicksSearch": u"Клики (поиск)",
            "CostSearch": u"Затраты (поиск)"
        },
        inplace=True)
    df_chart.index.names = [u"Баннер №"]

    return df_chart
