# coding: utf-8
"""xlsx-report:
  - 06_templates_status
  - 06_templates_incorrect_usage
  - 06_display_url
  - 06_wo_display_url

Help:
[HasTemplate]
  (https://yandex.ru/support/direct/features/ad-templates.xml?lang=en)
"""
from ..const import EXCEL_MAX_ROWS
from ..xlsx_chart.chart import Chart, chart_add_series
from ..utils import filter_by_adtype
import pandas as pd


def table_templates_status(df):
    """06_templates_status"""

    df_chart = filter_by_adtype(df, "TEXT_AD")
    df_chart = (df_chart[(df_chart.ShowsSearch > 0)]
                [["ShowsSearch", "ClicksSearch", "CostSearch",
                  "IsCorrectTemplate", "DirectBannerID", "DirectPhraseID"]]
                .assign(IsCorrectTemplate=lambda x: x.IsCorrectTemplate.astype(str))
                .groupby("IsCorrectTemplate", as_index=True)
                .agg({"ShowsSearch": pd.np.sum,
                      "ClicksSearch": pd.np.sum,
                      "CostSearch": pd.np.sum,
                      "DirectBannerID": lambda x: x.unique().size,
                      "DirectPhraseID": lambda x: x.unique().size})
                .fillna(0)
                .round(2)
                )
    if df_chart.shape[0] == 0:
        return None
    df_chart.rename(columns={
        "DirectBannerID": u"Кол-во баннеров",
        "DirectPhraseID": u"Кол-во фраз",
        "ShowsSearch": u"Показы (поиск)",
        "ClicksSearch": u"Клики (поиск)",
        "CostSearch": u"Затраты (поиск)"},
        inplace=True)
    df_chart.rename(index={
        "False": u"Используется фраза по-умолчанию",
        "True": u"Фраза подставляется",
        "None": u"Без шаблона"},
        inplace=True)
    df_chart.index.names = [u"Корректность"]

    return df_chart.loc[[u"Без шаблона", u"Используется фраза по-умолчанию", u"Фраза подставляется"]].fillna('')


def plot_templates_status(writer, sheet_name, df):
    """(ExcelWriter, str, DataFrame) -> ExcelWriter"""

    sheet = writer.sheets[sheet_name]

    chart = Chart(writer.book, "bar", "percent_stacked", u'Использование шаблона')
    chart_add_series(chart, 0, {
        'values': '=%s!B2:F2' % sheet_name,
        "categories": '=%s!B1:F1' % sheet_name,
        "name": "=%s!A2" % sheet_name,
        "data_labels": {"position": "inside_base"}})
    chart_add_series(chart, 1, {
        'values': '=%s!B3:F3' % sheet_name,
        "categories": '=%s!B1:F1' % sheet_name,
        "name": "=%s!A3" % sheet_name,
        "data_labels": {"position": "inside_end"}})
    chart_add_series(chart, 2, {
        'values': '=%s!B4:F4' % sheet_name,
        "categories": '=%s!B1:F1' % sheet_name,
        "name": "=%s!A4" % sheet_name,
        "data_labels": {"position": "inside_base"}})

    sheet.insert_chart('A7', chart)

    return writer


def table_templates_incorrect_usage(df):
    """06_templates_incorrect_usage"""

    df_chart = filter_by_adtype(df, "TEXT_AD")
    df_chart = (df_chart[
                  (df_chart.IsCorrectTemplate == False) &
                  (df_chart.ShowsSearch > 0)]  # noqa: E712
                .sort_values("ShowsSearch", ascending=False)
                .head(EXCEL_MAX_ROWS)
                .set_index("DirectBannerID")
                [["Title", "Body", "PhraseText", "ShowsSearch", "ClicksSearch", "CostSearch"]])
    df_chart.rename(index={
        "DirectBannerID": u"Баннер №"},
        inplace=True)
    df_chart.rename(columns={
        "Title": u"Заголовок",
        "Body": u"Объявление",
        "PhraseText": u"Рекламная фраза",
        "ShowsSearch": u"Показы (поиск)",
        "ClicksSearch": u"Клики (поиск)",
        "CostSearch": u"Затраты (поиск)"},
        inplace=True)
    df_chart.index.names = [u"Баннер №"]

    return df_chart


def table_display_url_has_template(df):
    """06_display_url"""

    df_chart = filter_by_adtype(df, "TEXT_AD")
    df_chart = (df_chart[(df_chart.ShowsSearch > 0)]
                [["ShowsSearch", "ClicksSearch", "CostSearch",
                  "DisplayUrlHasTemplate", "DirectBannerID", "DirectPhraseID"]]
                .assign(DisplayUrlHasTemplate=lambda x: x.DisplayUrlHasTemplate.astype(str))
                .groupby("DisplayUrlHasTemplate", as_index=True)
                .agg({"ShowsSearch": pd.np.sum,
                      "ClicksSearch": pd.np.sum,
                      "CostSearch": pd.np.sum,
                      "DirectBannerID": lambda x: x.unique().size,
                      "DirectPhraseID": lambda x: x.unique().size})
                .fillna(0)
                .round(2)
                )
    if df_chart.shape[0] == 0:
        return None
    df_chart.rename(columns={
        "DirectBannerID": u"Кол-во баннеров",
        "DirectPhraseID": u"Кол-во фраз",
        "ShowsSearch": u"Показы (поиск)",
        "ClicksSearch": u"Клики (поиск)",
        "CostSearch": u"Затраты (поиск)"},
        inplace=True)
    df_chart.rename(index={
        "False": u"Без шаблона",
        "True": u"С шаблоном",
        "None": u"Без отображаемой ссылки"},
        inplace=True)
    df_chart.index.names = [u"Использование отображаемой ссылки"]

    return df_chart.loc[[u"Без шаблона", u"С шаблоном", u"Без отображаемой ссылки"]].fillna('')


def table_wo_display_url(df):
    """06_wo_display_url"""

    df_chart = filter_by_adtype(df, "TEXT_AD")
    df_chart = (df_chart[
                  df_chart.DisplayUrlHasTemplate.isnull() &
                  (df_chart.ShowsSearch > 0)]  # noqa: E712
                .sort_values("ShowsSearch", ascending=False)
                .head(EXCEL_MAX_ROWS)
                .set_index("DirectBannerID")
                [["Title", "Body", "PhraseText", "ShowsSearch", "ClicksSearch", "CostSearch"]])
    if df_chart.shape[0] == 0:
        return None
    df_chart.rename(index={
        "DirectBannerID": u"Баннер №"},
        inplace=True)
    df_chart.rename(columns={
        "Title": u"Заголовок",
        "Body": u"Объявление",
        "PhraseText": u"Рекламная фраза",
        "ShowsSearch": u"Показы (поиск)",
        "ClicksSearch": u"Клики (поиск)",
        "CostSearch": u"Затраты (поиск)"},
        inplace=True)
    df_chart.index.names = [u"Баннер №"]

    return df_chart
