# coding: utf-8
"""xlsx-report:
  - 10_avg_time_target
  - 10_use_time_target_campaigns
  - 10_weighted_time_target

Help:
[Hourly CPC adjustment]
  (https://yandex.ru/support/direct/features/hourly-cpc-adjustment.xml?lang=en)
[Time targeting]
  (https://yandex.ru/support/direct/efficiency/timetargeting.xml?lang=en)
"""
from ..utils import extract_timetarget
import seaborn as sns
import pandas as pd


def table_avg_time_target(df):
    """10_avg_time_target"""

    df_chart = (df[["CampaignID", "CampaignName", "TimeTarget", "Shows", "Clicks", "Cost"]]
                .groupby(["CampaignID", "CampaignName", "TimeTarget"], as_index=False)
                .sum()
                .assign(TimeTarget=lambda x: x.TimeTarget.apply(extract_timetarget)))
    df_chart = ((df_chart.TimeTarget.sum() / df_chart.shape[0])
                .round(2)
                .rename(columns={0: u" День №"}))

    chart_avg_time_target(df_chart)
    if df_chart.shape[0] == 0:
        return None
    return df_chart


def chart_avg_time_target(df_chart):
    """10_avg_time_target"""
    sns.plt.figure()
    sns.heatmap(df_chart.iloc[:, 1:], cmap=sns.plt.cm.Paired_r, center=100, vmin=0,
                linewidths=.1, square=True, annot=True, fmt=".0f", cbar=False)
    sns.plt.suptitle(u"Усредненные корректировки временного таргетинга\n\n", y=.72, size=18)
    sns.plt.yticks(rotation=0)
    sns.plt.savefig("./charts/10_avg_time_target.png", dpi=200)


def use_time_target_campaigns(df):
    """10_use_time_target_campaigns"""

    chart_weighted_time_target(df)

    use_adjustment = lambda tt_cell: pd.np.concatenate(tt_cell.loc[:, 1:].values != 100).any()
    avg_adjustment = lambda tt_cell: pd.np.concatenate(tt_cell.loc[:, 1:].values).mean()

    df_chart = (df[["CampaignID", "CampaignName", "TimeTarget", "Shows", "Clicks", "Cost"]]
                .groupby(["CampaignID", "CampaignName", "TimeTarget"], as_index=False)
                .sum()
                .assign(TimeTarget=lambda x: x.TimeTarget.apply(extract_timetarget))
                .assign(use_adjustment=lambda x: x.TimeTarget.apply(use_adjustment))
                .assign(avg_adjustment=lambda x: x.TimeTarget.apply(avg_adjustment))
                .pipe(lambda df: df.loc[df.use_adjustment])
                .drop(["TimeTarget", "use_adjustment"], axis=1)
                .sort_values("CampaignID")
                .reset_index(drop=True))
    if df_chart.shape[0] == 0:
        return None
    df_chart.rename(columns={
        "CampaignID": u"Кампания №",
        "CampaignName": u"Название кампании",
        "avg_adjustment": u"Средняя корректировка",
        "Shows": u"Показы",
        "Clicks": u"Клики",
        "Cost": u"Затраты"},
        inplace=True)

    return df_chart


def chart_weighted_time_target(df):
    """10_weighted_time_target"""

    df_chart = (df[["CampaignID", "CampaignName", "TimeTarget", "Shows", "Clicks", "Cost"]]
                .groupby(["CampaignID", "CampaignName", "TimeTarget"], as_index=False)
                .sum()
                .assign(TimeTarget=lambda x: x.TimeTarget.apply(extract_timetarget) * x.Clicks))
    df_chart = ((df_chart.TimeTarget.sum() / df_chart.Clicks.sum())
                .round(2)
                .rename(columns={0: u" День, №"}))

    sns.plt.figure()
    sns.heatmap(df_chart.iloc[:, 1:], cmap=sns.plt.cm.Paired_r, center=100, vmin=0, linewidths=.1, square=True, annot=True, fmt=".0f", cbar=False)
    sns.plt.suptitle(u"Усредненные корректировки временного таргетинга\nвзвешенные на клики\n\n", y=.74, size=18)
    sns.plt.yticks(rotation=0)
    sns.plt.savefig("./charts/10_weighted_time_target.png", dpi=200)
