# coding: utf-8
"""xlsx-report:
  - 14_vcard_usage
  - 14_banners_wo_vcard

Help:
[vCard](https://yandex.com/support/direct/features/vcards.xml)
"""
from ..const import EXCEL_MAX_ROWS
from ..xlsx_chart.chart import Chart, chart_add_series
from ..utils import filter_by_adtype
import pandas as pd


def table_vcard_usage(df):
    """14_vcard_usage"""

    df_chart = filter_by_adtype(df, "TEXT_AD")
    df_chart = (df_chart[(df_chart.ShowsSearch > 0)]
                [["ShowsSearch", "ClicksSearch", "CostSearch",
                  "VisitCardUsage", "DirectBannerID", "DirectPhraseID"]]
                .assign(VisitCardUsage=lambda x: x.VisitCardUsage.astype(str))
                .groupby("VisitCardUsage", as_index=True)
                .agg({"ShowsSearch": pd.np.sum,
                      "ClicksSearch": pd.np.sum,
                      "CostSearch": pd.np.sum,
                      "DirectBannerID": lambda x: x.unique().size,
                      "DirectPhraseID": lambda x: x.unique().size})
                .fillna(0)
                .round(2)
                )
    if df_chart.shape[0] == 0:
        return None
    df_chart.rename(columns={
        "DirectBannerID": u"Кол-во баннеров",
        "DirectPhraseID": u"Кол-во фраз",
        "ShowsSearch": u"Показы (поиск)",
        "ClicksSearch": u"Клики (поиск)",
        "CostSearch": u"Затраты (поиск)"},
        inplace=True)
    df_chart.rename(index={
        "False": u"Без доп. полей",
        "True": u"С доп. полями",
        "None": u"Не используется"},
        inplace=True)
    df_chart.index.names = [u"Использование визитки"]

    return df_chart.loc[[u"Не используется", u"Без доп. полей", u"С доп. полями"]].fillna('')


def plot_vcard_usage(writer, sheet_name, df):
    """(ExcelWriter, str, DataFrame) -> ExcelWriter"""

    sheet = writer.sheets[sheet_name]

    chart = Chart(writer.book, "bar", "percent_stacked", u'Использование визитки')
    chart_add_series(chart, 0, {
        'values': '=%s!B2:F2' % sheet_name,
        "categories": '=%s!B1:F1' % sheet_name,
        "name": "=%s!A2" % sheet_name,
        "data_labels": {"position": "inside_base"}})
    chart_add_series(chart, 1, {
        'values': '=%s!B3:F3' % sheet_name,
        "categories": '=%s!B1:F1' % sheet_name,
        "name": "=%s!A3" % sheet_name,
        "data_labels": {"position": "inside_end"}})
    chart_add_series(chart, 2, {
        'values': '=%s!B4:F4' % sheet_name,
        "categories": '=%s!B1:F1' % sheet_name,
        "name": "=%s!A4" % sheet_name,
        "data_labels": {"position": "inside_base"}})

    sheet.insert_chart('A7', chart)

    return writer


def table_banners_wo_vcard(df):
    """14_banners_wo_vcard"""

    df_chart = filter_by_adtype(df, "TEXT_AD")
    df_chart = (df_chart[
                  df_chart.VisitCardUsage.isnull() &
                  (df_chart.ShowsSearch > 0)]  # noqa: E712
                .sort_values("ShowsSearch", ascending=False)
                .head(EXCEL_MAX_ROWS)
                .set_index("DirectBannerID")
                [["Title", "Body", "PhraseText", "ShowsSearch", "ClicksSearch", "CostSearch"]])
    if df_chart.shape[0] == 0:
        return None
    df_chart.rename(index={
        "DirectBannerID": u"Баннер №"},
        inplace=True)
    df_chart.rename(columns={
        "Title": u"Заголовок",
        "Body": u"Объявление",
        "PhraseText": u"Рекламная фраза",
        "ShowsSearch": u"Показы (поиск)",
        "ClicksSearch": u"Клики (поиск)",
        "CostSearch": u"Затраты (поиск)"},
        inplace=True)
    df_chart.index.names = [u"Баннер №"]

    return df_chart
