# coding: utf-8
"""xlsx-report:
  - 05_words_cnt
  - 13_top_words_usage
  - 13_top_words_uppercase
"""
from ..xlsx_chart.chart import Chart, chart_add_series, FONT_NAME
from ..utils import phrase_process, is_uppercase_word
from ..const import EXCEL_MAX_ROWS
import pandas as pd


def table_words_cnt(df, normalize):
    """05_words_cnt"""

    words_cnt = sorted(set(df.WordsCnt.unique()) | set(range(1, 8)))

    df_chart = (df[(
        df.ShowsSearch > 0
    )][["ShowsSearch", "ClicksSearch", "CostSearch", "BSPhraseID", "WordsCnt"]]
                .groupby("WordsCnt").agg({  # noqa: E126 (indent)
                    "ShowsSearch":
                    pd.np.sum,
                    "ClicksSearch":
                    pd.np.sum,
                    "CostSearch":
                    pd.np.sum,
                    "BSPhraseID":
                    lambda x: x.unique().size
                }))
    if df_chart.shape[0] == 0:
        return None
    df_chart.rename(
        columns={
            "BSPhraseID": u"Кол-во фраз",
            "ShowsSearch": u"Показы (поиск)",
            "ClicksSearch": u"Клики (поиск)",
            "CostSearch": u"Затраты (поиск)"
        },
        inplace=True)
    df_chart.index.name = u"Кол-во слов во фразе"

    df_chart = df_chart.loc[words_cnt].fillna(pd.np.nan)

    if normalize:
        df_chart = df_chart.apply(lambda x: x / x.sum())
        # chart_words_cnt(df_chart)

    return df_chart


def plot_words_cnt(writer, sheet_name, df):
    """(ExcelWriter, str, DataFrame) -> ExcelWriter"""

    sheet = writer.sheets[sheet_name]
    chart = Chart(writer.book, "column", None, u'Длина фраз')

    chart.set_y_axis({
        "name": u"Доля от общего",
        'name_font': {
            "size": 16,
            "bold": False,
            "name": FONT_NAME
        },
        'num_font': {
            "size": 14,
            "bold": False,
            "name": FONT_NAME
        },
        'num_format': '#0%',
        'major_tick_mark': 'none',
        'minor_tick_mark': 'none',
        'line': {
            'none': True
        }
    })

    cols = "BCDE"
    last_row = df.index.size + 1
    for n, col in enumerate(cols):
        chart_add_series(chart, n, {
            'values':
            '={0}!{1}2:{1}{2}'.format(sheet_name, col, last_row),
            "categories":
            '={}!A2:A{}'.format(sheet_name, last_row),
            "name":
            "={0}!{1}1".format(sheet_name, col),
            "data_labels": {
                "num_format": "#0%"
            }
        })

    sheet.insert_chart('A%s' % (last_row + 2), chart)

    return writer


# def chart_words_cnt(df_chart):
#     """05_words_cnt"""
#     sns.plt.figure()
#     (df_chart.plot.bar(rot=0))
#     sns.plt.suptitle(u"Длина фраз", size=18)
#     sns.plt.xlabel(u"Кол-во слов", size=16)
#     sns.plt.ylabel(u"Доля от общего", size=16)
#     sns.plt.savefig("./charts/05_words_cnt.png", dpi=200)


def table_top_words_usage(df):
    """13_top_words_usage"""

    col_stat = [
        "Shows", "Clicks", "Cost", "ShowsSearch", "ClicksSearch", "CostSearch",
        "ShowsRSYA", "ClicksRSYA", "CostRSYA"
    ]
    df_chart = df[["PhraseText"] + col_stat].copy()
    df_chart.PhraseText = df_chart.PhraseText.apply(
        lambda x: map(lambda x: x.strip(), phrase_process(x).split(" ")))

    df_chart = pd.np.concatenate(map(lambda (_, row): [row.set_value("PhraseText", w).tolist() for w in row.PhraseText], df_chart.iterrows()))
    df_chart = pd.DataFrame(df_chart, columns=["Word"] + col_stat)
    df_chart[col_stat] = df_chart[col_stat].astype(float)
    df_chart = df_chart.groupby("Word").sum().sort_values(
        "Clicks", ascending=False).head(EXCEL_MAX_ROWS)
    df_chart["OnlyRSYA"] = df_chart["ShowsSearch"] == 0.0
    df_chart["OnlySearch"] = df_chart["ShowsRSYA"] == 0.0
    df_chart = df_chart.round(2)

    df_chart.rename(
        columns={
            "Shows": u"Показы",
            "Clicks": u"Клики",
            "Cost": u"Затраты",
            "ShowsSearch": u"Показы (поиск)",
            "ClicksSearch": u"Клики (поиск)",
            "CostSearch": u"Затраты (поиск)",
            "ShowsRSYA": u"Показы (сети)",
            "ClicksRSYA": u"Клики (сети)",
            "CostRSYA": u"Затраты (сети)",
            "OnlySearch": u"Только поиск",
            "OnlyRSYA": u"Только сети"
        },
        inplace=True)
    df_chart.index.names = [u"Слово из фразы"]

    return df_chart


def table_top_words_uppercase(df):
    """13_top_words_uppercase"""

    col_stat = [
        "Shows", "Clicks", "Cost", "ShowsSearch", "ClicksSearch", "CostSearch",
        "ShowsRSYA", "ClicksRSYA", "CostRSYA"
    ]
    df_chart = df[["Title", "Body"] + col_stat].copy()
    df_chart["BannerText"] = df_chart.apply(
        lambda row: u"%s %s" % (row["Title"], row["Body"]), axis=1)
    df_chart.BannerText = df_chart.BannerText.apply(lambda x: map(lambda x: x.strip(), phrase_process(x, to_lower=False).split(" ")))
    df_chart.drop(["Title", "Body"], inplace=True, axis=1)
    df_chart = pd.np.concatenate(map(lambda (_, row): [row.set_value("BannerText", w).tolist() for w in row.BannerText], df_chart.iterrows()))
    df_chart = pd.DataFrame(df_chart, columns=col_stat + ["Word"])
    df_chart = df_chart[df_chart["Word"].apply(is_uppercase_word)]

    if df_chart.shape[0] == 0:
        return None

    df_chart[col_stat] = df_chart[col_stat].astype(float)
    df_chart = df_chart.groupby("Word").sum().sort_values(
        "Clicks", ascending=False).head(EXCEL_MAX_ROWS)
    df_chart["OnlyRSYA"] = df_chart["ShowsSearch"] == 0.0
    df_chart["OnlySearch"] = df_chart["ShowsRSYA"] == 0.0
    df_chart = df_chart.round(2)

    df_chart.rename(
        columns={
            "Shows": u"Показы",
            "Clicks": u"Клики",
            "Cost": u"Затраты",
            "ShowsSearch": u"Показы (поиск)",
            "ClicksSearch": u"Клики (поиск)",
            "CostSearch": u"Затраты (поиск)",
            "ShowsRSYA": u"Показы (сети)",
            "ClicksRSYA": u"Клики (сети)",
            "CostRSYA": u"Затраты (сети)",
            "OnlySearch": u"Только поиск",
            "OnlyRSYA": u"Только сети"
        },
        inplace=True)
    df_chart.index.names = [u"Слово из баннера"]

    return df_chart
