# coding: utf-8
"""xlsx-search_queries"""
from .queries import (
    table_highlight_queries,
    plot_highlight_queries_hist,
    table_top_nonbolded_queries,
    table_top_intersection_by_queries
)
from .nested_queries import (
    shows_by_words_cnt,
    shows_by_nesting_type,
    new_clicks_by_words_cnt,
    ctr_cpc_from_query_len,
    top_queries_from_nested,
    total_clicks_from_nested,
    total_cost_from_nested
)
from ..utils import note_maker as note, add_sheet_to_xlsx


def make_builder(fn, sheet_name, sheet_fmt=None, **kw_args):
    """((DataFrame) -> DataFrame, str, [(ExcelWriter, str, DataFrame) -> ExcelWriter], **kw_args) ->
       ((ExcelWriter, DataFrame) -> ExcelWriter)"""
    if sheet_fmt is None:
        sheet_fmt = []
    if not isinstance(sheet_fmt, list):
        raise TypeError("`sheet_fmt` must be iterable or None")

    def builder(logger, writer, df):
        add_sheet_to_xlsx(logger, writer, fn(df), sheet_name=sheet_name, sheet_fmt=sheet_fmt, **kw_args)

    return builder


# pd.option_context('display.max_colwidth', 200)

xlsx_search_queries = [
    make_builder(
        lambda df: table_highlight_queries(df, normalize=False),
        "01_highlight_hist_queries",
        startcol=7),
    make_builder(
        lambda df: table_highlight_queries(df, normalize=True),
        "01_highlight_hist_queries",
        sheet_fmt=[
            plot_highlight_queries_hist,
            note(u"Подсветка - доля слов от поискового запроса в объявлении: заголовок + тело баннера")]),
    make_builder(
        table_top_nonbolded_queries,
        "01_top_non-bolded_queries",
        sheet_fmt=[
            note(u"Подсветка - доля слов от поискового запроса в объявлении: заголовок + тело баннера")],
        index=None),
    make_builder(
        shows_by_words_cnt,
        "02_nested_queries",
        sheet_fmt=[note(u"Показы по количеству слов")]),
    make_builder(
        shows_by_nesting_type,
        "02_nested_queries",
        sheet_fmt=[note(u"Показы по количеству слов в поисковом запросе и типу соответствия", cell="K1")],
        startcol=10),
    make_builder(
        new_clicks_by_words_cnt,
        "02_nested_queries",
        sheet_fmt=[note(u"Новые клики по количеству <слов в условии показа>(строки) на <слова в поисковом запросе>(столбцы). Только для неоптимальных по длине показов.", cell="A13")],
        startrow=12),
    make_builder(
        total_clicks_from_nested,
        "02_nested_queries",
        sheet_fmt=[note(u"Фактические и новые клики от проработки вложенных запросов.", cell="K13")],
        startrow=12,
        startcol=10),
    make_builder(
        ctr_cpc_from_query_len,
        "02_nested_queries",
        sheet_fmt=[note(u"CTR, CPC по количеству <слов в поисковом запросе>. Только для неоптимальных по длине показов.", cell="A23")],
        startrow=22),
    make_builder(
        total_cost_from_nested,
        "02_nested_queries",
        sheet_fmt=[note(u"Фактические и новые затраты от проработки вложенных запросов", cell="N13")],
        startrow=12,
        startcol=13),
    make_builder(
        top_queries_from_nested,
        "02_top_nested_queries",
        sheet_fmt=[note(u"Топ запросов по количеству новых кликов")],
        index=None),
    make_builder(
        lambda df: table_top_intersection_by_queries(df, match_type="fuzzy"),
        "03_top_intersection_by_queries",
        sheet_fmt=[note(u"Топ фраз пересекающихся по показам, но не дубли. > 50 показов. \nФразы пересекаются, если у них есть показы по одинаковым запросам. \nРегиональность и временной таргетинг не учитываются.\nОтранжировано по количесву запросов.")],
        index=None),
    make_builder(
        lambda df: table_top_intersection_by_queries(df, match_type="exact"),
        "03_top_doubles_by_queries",
        sheet_fmt=[note(u"Топ одинаковых фраз пересекающихся по показам. > 50 показов. \nФразы пересекаются, если у них есть показы по одинаковым запросам. \nРегиональность и временной таргетинг не учитываются.\nОтранжировано по количесву запросов.")],
        index=None),
]
