# coding: utf-8
"""xlsx-queries:
  - 01_top_non-bolded_queries
  - 01_highlight_hist_queries
  - 02_nested_queries
  - 02_top_nested_queries
"""
from ..const import EXCEL_MAX_ROWS
import pandas as pd


def shows_by_words_cnt(df):
    "02_nested_queries"

    col1 = df.groupby(["PhraseWordsCnt"]).agg({"Shows": pd.np.sum})[["Shows"]]
    col1 = col1.loc[range(1, 8)]  # на случай когда не все длины есть
    col1.index.rename(u"Кол-во слов", inplace=True)
    col1.rename(columns={"Shows": u"Рекламные фразы"}, inplace=True)

    col2 = df.groupby(["QueryWordsCnt"]).agg({"Shows": pd.np.sum})[["Shows"]]
    col2.index.rename("WordsCnt", inplace=True)
    col2.rename(columns={"Shows": u"Поисковые запросы"}, inplace=True)

    return col1.join(col2).fillna(0)


def shows_by_nesting_type(df):
    "02_nested_queries"

    df_chart = df.loc[:, ["Shows", "PhraseWordsCnt", "QueryWordsCnt"]].copy()
    df_chart["isExactNesting"] = df_chart.apply(
        lambda fr: fr["QueryWordsCnt"] == fr["PhraseWordsCnt"],
        axis=1)
    df_chart = (df_chart
                .groupby(["QueryWordsCnt", "isExactNesting"])
                .agg({"Shows": pd.np.sum})
                .unstack("isExactNesting")
                ["Shows"]
                .fillna(0)
                .rename(columns={True: u"Точное", False: u"Широкое"}))
    df_chart.index.rename(u"Кол-во слов", inplace=True)
    df_chart.columns.rename("", inplace=True)

    total_shows = float(df_chart.values.sum())
    df_chart[u"Неоптимально"] = df_chart[u"Широкое"].apply(lambda x: x / total_shows)

    # на случай когда не все длины есть
    return df_chart.loc[range(1, 8)].fillna(0)


def new_clicks_by_words_cnt(df):
    "02_nested_queries"

    df_chart = (df.loc[
        df["QueryWordsCnt"] > df["PhraseWordsCnt"],
        ["NewClicksFromNested", "PhraseWordsCnt", "QueryWordsCnt"]]
        .groupby(["PhraseWordsCnt", "QueryWordsCnt"])
        .agg({"NewClicksFromNested": pd.np.sum})
        ["NewClicksFromNested"]
        .unstack("QueryWordsCnt"))

    df_chart.index.rename(u"Кол-во слов", inplace=True)

    # на случай когда не все длины есть
    # ограничено 6 словами, так как если у фразы 7 слов, то
    # прорабатывать нечего
    return df_chart.loc[range(1, 7)]


def total_clicks_from_nested(df):
    "02_nested_queries"

    df_chart = df.loc[:, [
            "QueryWordsCnt",
            "PhraseWordsCnt",
            "Clicks",
            "NewClicksFromNested"]
        ].copy()
    df_chart["isExactNesting"] = df_chart.apply(
        lambda fr: fr["QueryWordsCnt"] == fr["PhraseWordsCnt"],
        axis=1)
    df_chart = df_chart.groupby("isExactNesting")[["Clicks", "NewClicksFromNested"]].sum().T
    df_chart.loc["NewClicksFromNested", True] = 0
    df_chart = df_chart.unstack().sum(level=1)
    df_chart.rename(index={
        "NewClicksFromNested": u"Новые",
        "Clicks": u"Фактические"},
        inplace=True)

    return pd.DataFrame(df_chart, columns=[u"Клики"]).loc[[u"Фактические", u"Новые"]]


def total_cost_from_nested(df):
    "02_nested_queries"

    df_chart = df.loc[:, [
            "QueryWordsCnt",
            "PhraseWordsCnt",
            "Cost",
            "NewCostFromNested"]
        ].copy()
    df_chart["isExactNesting"] = df_chart.apply(
        lambda fr: fr["QueryWordsCnt"] == fr["PhraseWordsCnt"],
        axis=1)
    df_chart = df_chart.groupby("isExactNesting")[["Cost", "NewCostFromNested"]].sum().T
    df_chart.loc["NewCostFromNested", True] = 0
    df_chart = df_chart.unstack().sum(level=1)
    df_chart.rename(index={
        "NewCostFromNested": u"Новые",
        "Cost": u"Фактические"},
        inplace=True)

    return pd.DataFrame(df_chart, columns=[u"Затраты"]).loc[[u"Фактические", u"Новые"]]


def ctr_cpc_from_query_len(df):
    "02_nested_queries"
    dims = ["QueryWordsCnt"]
    metricks = ["NewClicksFromNested", "Clicks", "Cost", "NewCostFromNested", "Shows"]
    df_chart = (df.loc[
        df["QueryWordsCnt"] > df["PhraseWordsCnt"],
        metricks + dims]
        .groupby("QueryWordsCnt")
        [metricks].sum()
    )
    df_chart[u"Текущий CTR"] = df_chart["Clicks"] / df_chart["Shows"]
    df_chart[u"Прогнозный CTR"] = (df_chart["Clicks"] + df_chart["NewClicksFromNested"]) / df_chart["Shows"]
    df_chart[u"Текущий CPC"] = df_chart["Cost"] / df_chart["Clicks"]
    df_chart[u"Прогнозный CPC"] = (df_chart["Cost"] + df_chart["NewCostFromNested"]) / (df_chart["Clicks"] + df_chart["NewClicksFromNested"])

    df_chart.index.rename(u"Кол-во слов", inplace=True)

    cols = [u"Текущий CTR", u"Прогнозный CTR", u"Текущий CPC", u"Прогнозный CPC"]

     # на случай когда не все длины есть
     # от 2, тк запросы с длиной 1 всегда оптимальны
    return df_chart[cols].round(2).loc[range(2, 8)]


def top_queries_from_nested(df):
    "02_top_nested_queries"

    cols = ["CampaignID", "CampaignName", "DirectBannerID", "Query",
            "PhraseText", "QueryWordsCnt", "PhraseWordsCnt",
            "Shows", "Clicks", "Cost",
            "NewClicksFromNested", "NewCPCFromNested", "NewCostFromNested"]
    df_chart = (df.loc[
        df["QueryWordsCnt"] > df["PhraseWordsCnt"],
        cols]
        .sort_values("NewClicksFromNested", ascending=False)
        .head(EXCEL_MAX_ROWS))

    df_chart.rename(columns={
        "CampaignID": u"Кампания №",
        "CampaignName": u"Кампания",
        "DirectBannerID": u"Баннер №",
        "Query": u"Запрос",
        "PhraseText": u"Условие показа",
        "QueryWordsCnt": u"Длина запроса",
        "PhraseWordsCnt": u"Длина условия показа",
        "Shows": u"Показы",
        "Clicks": u"Клики",
        "Cost": u"Затраты",
        "NewClicksFromNested": u"Добавленные клики",
        "NewCostFromNested": u"Добавленные затраты"
    }, inplace=True)
    col_ord = [u"Кампания №", u"Кампания", u"Баннер №", u"Запрос",
               u"Условие показа", u"Длина запроса", u"Длина условия показа",
               u"Показы", u"Клики", u"Затраты",
               u"Добавленные клики", u"Добавленные затраты"]

    return df_chart.loc[:, col_ord]
