# coding: utf-8
"""xlsx-queries:
  - 01_top_non-bolded_queries
  - 01_highlight_hist_queries
"""
from ..xlsx_chart.chart import Chart, chart_add_series, FONT_NAME
from ..utils import search_bid
from ..const import EXCEL_MAX_ROWS
import pandas as pd


def table_top_nonbolded_queries(df):
    """01_top_nonbolded_queries"""

    df_chart = df[["DirectBannerID", "Title", "Body", "Query", "PhraseText", "QueryHighlighTitleBody",
                   "QueryNonBoldedWordsTitleBody", "QueryWordsCnt", "Shows", "Clicks", "Cost"]].copy()
    df_chart = df_chart[(df_chart.QueryWordsCnt != 0) & (df_chart.Shows > 0)]

    if df_chart.shape[0] == 0:
        return None

    df_chart["QueryBoldShare"] = df_chart.QueryHighlighTitleBody / df_chart.QueryWordsCnt
    df_chart.drop(["QueryHighlighTitleBody", "QueryWordsCnt"], axis=1, inplace=True)
    df_chart['ShowsNonBolded'] = ((1 - df_chart['QueryBoldShare']) * df_chart['Shows']).astype(int)
    df_chart = df_chart.sort_values("ShowsNonBolded", ascending=False).head(EXCEL_MAX_ROWS)
    df_chart["HelperURL"] = df_chart.DirectBannerID.apply(lambda x: search_bid(x))
    df_chart = df_chart[["Title", "Body", "Query", "PhraseText", "QueryNonBoldedWordsTitleBody",
                         "Shows", "Clicks", "Cost", "HelperURL"]]
    df_chart.rename(columns={
        "Title": u"Заголовок",
        "Body": u"Объявление",
        "Query": u"Поисковый запрос",
        "PhraseText": u"Условие показа",
        "QueryNonBoldedWordsTitleBody": u"Неподсвеченные слова",
        "Shows": u"Показы",
        "Clicks": u"Клики",
        "Cost": u"Затраты",
        "HelperURL": u"Ссылка на баннер"},
        inplace=True)

    return df_chart[[u"Заголовок", u"Объявление", u"Поисковый запрос", u"Условие показа", u"Неподсвеченные слова", u"Показы", u"Клики", u"Затраты"]]


def table_highlight_queries(df, normalize):
    """01_highlight_hist_queries"""

    df_chart = df[["QueryHighlighTitleBody", "QueryWordsCnt", "Shows", "Clicks", "Cost", "DirectBannerID"]].copy()
    df_chart["QueryBoldShare"] = df_chart.QueryHighlighTitleBody / df_chart.QueryWordsCnt
    df_chart = df_chart[df_chart.Shows > 0]
    if df_chart.shape[0] == 0:
        df_chart = None

    df_chart.QueryBoldShare = (df_chart.QueryBoldShare * 100).round(-1).astype(int).values
    df_chart = (df_chart.groupby(["QueryBoldShare"], as_index=True)
                .agg({"Shows": pd.np.sum,
                      "Clicks": pd.np.sum,
                      "Cost": pd.np.sum,
                      "DirectBannerID": lambda x: x.unique().size}))

    df_chart.rename(columns={
        "DirectBannerID": u"Кол-во баннеров",
        "Shows": u"Показы",
        "Clicks": u"Клики",
        "Cost": u"Затраты"},
        inplace=True)
    df_chart.index.names = [u"Подсветка"]
    df_chart = df_chart.loc[range(0, 110, 10)].fillna(pd.np.nan)

    if normalize:
        df_chart = df_chart.apply(lambda x: x / x.sum())

    df_chart = df_chart[[u"Кол-во баннеров", u"Показы", u"Клики", u"Затраты"]]

    return df_chart


def plot_highlight_queries_hist(writer, sheet_name, df):
    """
    (ExcelWriter, str, DataFrame) -> ExcelWriter
    01_highlight_hist_queries
    """

    sheet = writer.sheets[sheet_name]
    chart = Chart(writer.book, "column", None, u'Подсветка поисковых запросов')
    chart.set_y_axis({"name": u"Доля от общего",
                      'name_font': {"size": 16, "bold": False, "name": FONT_NAME},
                      'num_font': {"size": 14, "bold": False, "name": FONT_NAME},
                      'num_format': '#0%',
                      'major_tick_mark': 'none',
                      'minor_tick_mark': 'none',
                      'line': {'none': True}})
    cols = "BCDE"
    last_row = df.index.size + 2
    for n, col in enumerate(cols):
        chart_add_series(chart, n, {
            'values': '={0}!{1}2:{1}{2}'.format(sheet_name, col, last_row),
            "categories": '={}!A2:A{}'.format(sheet_name, last_row),
            "name": "={0}!{1}1".format(sheet_name, col),
            "data_labels": {"value": False}})

    sheet.insert_chart('A%s' % (last_row + 2), chart)

    return writer


def table_top_intersection_by_queries(df, match_type, min_shows=50):
    """03_top_intersection"""
    assert match_type in ("fuzzy", "exact"), "Unknown `match_type`: %s" % match_type
    df_j = pd.merge(df, df, on="Query", how="inner")
    # Все физически возможные пересечения пересечения
    df_j = df_j.loc[
        (
            (df_j.PhraseText_x < df_j.PhraseText_y) &
            (df_j.DirectBannerID_x != df_j.DirectBannerID_y)) |
        (
            (df_j.PhraseText_x == df_j.PhraseText_y) &
            (df_j.DirectBannerID_x < df_j.DirectBannerID_y))]
    # Фильтрация в зависимости от параметров - дубли или все кроме дублей
    if match_type == "exact":
        df_j = df_j.loc[(df_j.PhraseText_x == df_j.PhraseText_y)]
    elif match_type == "fuzzy":
        df_j = df_j.loc[(df_j.PhraseText_x != df_j.PhraseText_y)]
    df_j["Count"] = 1
    df_j = df_j.groupby([
        "PhraseText_x", "PhraseText_y",
        "DirectBannerID_x", "DirectBannerID_y",
        "CampaignID_x", "CampaignID_y",
        "CampaignName_x", "CampaignName_y"],
        as_index=False).agg(
        {
            "Shows_x": sum,
            "Shows_y": sum,
            "Clicks_x": sum,
            "Clicks_y": sum,
            "Cost_x": sum,
            "Cost_y": sum,
            "Count": sum
        }
    )
    df_j = df_j.loc[(df_j.Shows_x + df_j.Shows_y) > min_shows]
    df_j = df_j.sort_values("Count", ascending=False).reset_index()
    df_j = df_j[[
        "PhraseText_x", "PhraseText_y",
        "DirectBannerID_x", "DirectBannerID_y",
        "CampaignID_x", "CampaignID_y",
        "CampaignName_x", "CampaignName_y",
        "Shows_x", "Shows_y",
        "Clicks_x", "Clicks_y",
        "Cost_x", "Cost_y",
        "Count"]]
    df_j.rename(columns={
        "PhraseText_x": u"Условие показа X",
        "PhraseText_y": u"Условие показа Y",
        "DirectBannerID_x": u"Баннер № X",
        "DirectBannerID_y": u"Баннер № Y",
        "CampaignID_x": u"Кампания X",
        "CampaignID_y": u"Кампания Y",
        "CampaignName_x": u"Кампания (имя) X",
        "CampaignName_y": u"Кампания (имя) Y",
        "Shows_x": u"Показы X",
        "Shows_y": u"Показы Y",
        "Clicks_x": u"Клики X",
        "Clicks_y": u"Клики Y",
        "Cost_x": u"Затраты X",
        "Cost_y": u"Затраты Y",
        "Count": u"Кол-во запросов"
    }, inplace=True)

    return df_j.head(EXCEL_MAX_ROWS)
